import logging
from functools import wraps

import aiohttp.web

from aiojson.response import BadResponse, GoodResponse
from aiojson.exception import ApiException

routes = aiohttp.web.RouteTableDef()

log = logging.getLogger("aiojson." + __name__)


def route(method, path, **kwargs):
    def func_wrap(func):
        async def arg_wrap(*inner_args, **inner_kwargs):
            try:
                result = await func(*inner_args, **inner_kwargs)
                if isinstance(result, aiohttp.web.Response):
                    return result
                return GoodResponse(result)
            except ApiException as e:
                return BadResponse(e.message, status=e.status)
            except Exception as e:
                log.exception(e)
                return BadResponse("Something went wrong. Please try again later")

        func_wrap.__dict__ = arg_wrap.__dict__
        func_wrap.__name__ = arg_wrap.__name__
        func_wrap.__doc__ = arg_wrap.__doc__
        func_wrap.__defaults__ = arg_wrap.__defaults__
        return method(path, **kwargs)(arg_wrap)
    return func_wrap


def get(path: str, **kwargs):
    return route(routes.get, path, **kwargs)


def post(path: str, **kwargs):
    return route(routes.post, path, **kwargs)


def delete(path: str, **kwargs):
    return route(routes.delete, path, **kwargs)


def put(path: str, **kwargs):
    return route(routes.put, path, **kwargs)


def patch(path: str, **kwargs):
    return route(routes.patch, path, **kwargs)


def head(path: str, **kwargs):
    return route(routes.head, path, **kwargs)


def view(path: str, **kwargs):
    return route(routes.view, path, **kwargs)



import aiohttp.web
import pytest

from aiojson import routes
from aiojson.exception import ApiException


@pytest.mark.parametrize("method", [
    routes.get,
    routes.post,
    routes.patch,
    routes.put,
    routes.delete]
)
async def test_good(aiohttp_client, method):

    routes.routes._items.clear()

    @method("/test")
    async def test(request):
        return {"interfaces": ["en2"]}

    app = aiohttp.web.Application()
    app.router.add_routes(routes.routes)
    client = await aiohttp_client(app)
    resp = await getattr(client, method.__name__)('/test')
    assert resp.status == 200
    body = await resp.json()
    assert {"error": False, "result": {"interfaces": ["en2"]}} == body


@pytest.mark.parametrize("method", [
    routes.get,
    routes.post,
    routes.patch,
    routes.put,
    routes.delete],
)
@pytest.mark.parametrize("exception", [ApiException, Exception])
async def test_bad_exceptions(aiohttp_client, method, exception):

    routes.routes._items.clear()

    error_message = {"missing_fields": "test", "text": "Some required fields are missing"}

    @method("/test")
    async def test(request):
        raise exception(error_message)

    app = aiohttp.web.Application()
    app.router.add_routes(routes.routes)
    client = await aiohttp_client(app)
    resp = await getattr(client, method.__name__)('/test')
    assert resp.status == 400 if exception == Exception else 500
    body = await resp.json()
    if exception == Exception:
        assert {"error": True, "reason": "Something went wrong. Please try again later"} == body
    else:
        assert {"error": True, "reason": error_message} == body



import json
from aiohttp import web


def json_error(message, code, status):
    return web.Response(
        body=json.dumps({
            "error": True,
            "reason": {
                "text": message,
                "code": code
            }}).encode("utf-8"),
        content_type="application/json", status=status)


async def error_middleware(app, handler):
    async def middleware_handler(request):
        try:
            response = await handler(request)
            if response.status == 404:
                return json_error(response.message, "NotFound", response.status)
            return response
        except web.HTTPException as ex:
            if ex.status == 404:
                return json_error(ex.reason, "NotFound", ex.status)
            raise ex
    return middleware_handler
